package com.sociallangoliers.config;

import java.util.*;

import org.apache.commons.lang.BooleanUtils;
import org.apache.commons.lang.math.NumberUtils;
import twitter4j.auth.RequestToken;

/**
 * Created with IntelliJ IDEA.
 * User: user
 * Date: 1/15/13
 * Time: 11:26 PM
 */
public class TwitterConfig {
    public static enum TwitterActions {
        Tweets(1),
        Retweets(2),
        Favorites(3),
        DirectMessages(4),
        Lists(5),
        Friendships(6);
        int id;

        TwitterActions(int value) {
            id = value;

        }

        public int getId() {
            return id;
        }
    }

    private Map<TwitterActions, Boolean> twitterActions = new HashMap<TwitterActions, Boolean>();

    private RequestToken token;
    private int incrementCount = 0;

    public TwitterConfig(Map values) {
        loadTwitterConfig(values);

    }
        /**
	 * Load data specific to twitter into memory.
	 */
	private void loadTwitterConfig(Map prop) {

        String consumer = null;
        String secret = null;

        if (prop.containsKey("twitter.consumer"))
            consumer = (String) prop.get("twitter.consumer");

        if (prop.containsKey("twitter.secret"))
            secret = (String) prop.get("twitter.secret");

        if (secret != null && consumer != null) {
            this.token = new RequestToken(consumer, secret);

        }

        if (prop.get("twitter.incrementCount") != null)
            incrementCount = NumberUtils.toInt((String) prop.get("twitter.incrementCount"), 199);

        twitterActions.put(TwitterActions.DirectMessages, BooleanUtils.toBoolean((String) prop.get("twitter.DirectMessages")));
        twitterActions.put(TwitterActions.Lists, BooleanUtils.toBoolean((String) prop.get("twitter.Lists")));
        twitterActions.put(TwitterActions.Tweets, BooleanUtils.toBoolean((String) prop.get("twitter.Tweets")));
        twitterActions.put(TwitterActions.Favorites, BooleanUtils.toBoolean((String) prop.get("twitter.Favorites")));
        twitterActions.put(TwitterActions.Retweets, BooleanUtils.toBoolean((String) prop.get("twitter.Retweets")));
        twitterActions.put(TwitterActions.Friendships, BooleanUtils.toBoolean((String) prop.get("twitter.Friendships")));

    }

	public boolean isCleanDirectMessages() {
		return (twitterActions.containsKey(TwitterActions.DirectMessages))  ? twitterActions.get(TwitterActions.DirectMessages) : false;
	}

	public boolean isCleanFavorites() {
		return (twitterActions.containsKey(TwitterActions.Favorites))  ? twitterActions.get(TwitterActions.Favorites) : false;
	}

	public boolean isCleanLists() {
		return (twitterActions.containsKey(TwitterActions.Lists))  ? twitterActions.get(TwitterActions.Lists): false;
	}

	public boolean isCleanFriendships() {
		return (twitterActions.containsKey(TwitterActions.Friendships))  ? twitterActions.get(TwitterActions.Friendships): false;
	}

	public boolean isCleanTweets() {
		return (twitterActions.containsKey(TwitterActions.Tweets))  ? twitterActions.get(TwitterActions.Tweets) : false;
	}

	public boolean isCleanReTweets() {
		return (twitterActions.containsKey(TwitterActions.Retweets))  ? twitterActions.get(TwitterActions.Retweets) : false;
	}


	public int getIncrementCount() {
		return incrementCount;
	}


	public RequestToken getToken() {
		return token;
	}


}
