package com.sociallangoliers.socialservices;


import org.apache.commons.lang.WordUtils;
import org.apache.log4j.PropertyConfigurator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;
import java.util.Scanner;

public class Main {
    Map<String, PruneSocial> socialServices = new HashMap();

    enum SocialCode {
        twitter,
        facebook,
        reddit,
        linkedin,
        foursquare,
        instagram;
    }

    static Logger logger = LoggerFactory.getLogger(Main.class);

    // Static constructor
    static {
        PropertyConfigurator.configure(Twitter.class.getResource("/log4j.properties"));
    }


    /**
     * Give a numeric choice matching the enum, it will instantiate a corresponding object.
     * @param choice
     * @return
     */
    public PruneSocial instantiate(int choice) {
        SocialCode oChoice = null;
        SocialCode[] possibleValues = SocialCode.facebook.getDeclaringClass().getEnumConstants();
        for( SocialCode value : possibleValues) {
            if(value.ordinal() == choice) {
                oChoice = value;
                break;
            }
        }
        if(socialServices.containsKey(oChoice)) {
            PruneSocial ins = socialServices.get(oChoice);
            if( ins != null) {
                return ins;
            }
        }
        try {
            PruneSocial ins = (PruneSocial) Class.forName("com.sociallangoliers.socialservices." + WordUtils.capitalize(oChoice.name())).newInstance();
            socialServices.put(oChoice.toString(), ins);
            return ins;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Uses reflection of enum to generate a menu and read in user choice.
     * @return
     */
    private int printMenu() {
        logger.info("Which social media service would you like to use? ");
        SocialCode[] possibleValues = SocialCode.facebook.getDeclaringClass().getEnumConstants();
         for( SocialCode value : possibleValues) {
             logger.info( value.ordinal() + ". " + WordUtils.capitalize(value.name()));
         }
        logger.info("99. Exit");
        System.out.print("What is your choice?  ==> ");
        Scanner scanner = new Scanner(System.in);
        int choice = scanner.nextInt();
        return choice;

    }

    /**
     * method will read the user's choice given by printMenu and execute the instantiated object
     *
     */
    public void processChoices() {

        while (true) {
            int choice = printMenu();
            PruneSocial obj = instantiate(choice);
            if (obj == null) {
                logger.error("Could not instantiate object for choice entered.  Exiting...");
                System.exit(choice);
            }
            obj.execute();
        }

    }


    public static void main(String[] args) {
        Main main = new Main();
        main.processChoices();


    }
}
